/*
   Calcula as fatias para cada processo pai, respeitando o máximo uso de
   processingLimit por CPU.
   
   Processos pai com mais filhos não podem ganhar mais fatias, a menos que
   não sejam utilizadas por outros.
*/

#include <stdlib.h>
#include "calcSlices.h"
#include "macros.h"

sliceListT createSliceList(short int nCPUs, char limitPerCPU, unsigned long period)
{
    sliceListT list = malloc(sizeof(*list));
    list->nCPUs = nCPUs;
    list->limitPerCPU = limitPerCPU;
    list->period = period;
    list->nApps = 0;
    return list;
}

void setSliceListParams(sliceListT list, short int nCPUs, char limitPerCPU)
{
    list->nCPUs = nCPUs;
    list->limitPerCPU = limitPerCPU;
}

void setSliceListNTasks(sliceListT list, int pid, unsigned char nTasks)
{
    int i;

    for (i = 0; i < list->nApps; i++)
        if (list->pid[i] == pid)
            list->nTasks[i] = nTasks;
}

void addAppToSliceList(sliceListT list, int pid, unsigned char nTasks)
{
    if (list->nApps == MAX_APPS)
        return;
    list->pid[list->nApps] = pid;
    list->nTasks[list->nApps] = nTasks;
    list->nApps++;
}

void removeAppFromSliceList(sliceListT list, int pid)
{
    int i;

    for (i = 0; i < list->nApps && list->pid[i] != pid; i++);
    if (list->pid[i] == pid)
        list->nApps--;
    for (; i < list->nApps; i++) {
        list->pid[i] = list->pid[i + 1];
        list->nTasks[i] = list->nTasks[i + 1];
        list->slice[i] = list->slice[i + 1];
    }
}

unsigned long getSliceFromSliceList(sliceListT list, int pid)
{
    int i;

    for (i = 0; i < list->nApps; i++)
        if (list->pid[i] == pid)
            return list->slice[i];

    return 0;
}

void swapSliceListApps(sliceListT list, int i, int j)
{
    int pid;
    short int nTasks;
    unsigned long slice;

    pid = list->pid[i];
    nTasks = list->nTasks[i];
    slice = list->slice[i];

    list->pid[i] = list->pid[j];
    list->nTasks[i] = list->nTasks[j];
    list->slice[i] = list->slice[j];

    list->pid[j] = pid;
    list->nTasks[j] = nTasks;
    list->slice[j] = slice;
}

/* Selection sort */
void sortSliceListByTasks(sliceListT list)
{
    int i, j, min;

    for (i = 0; i < list->nApps - 1; i++) {
        min = i;
        for (j = i + 1; j < list->nApps; j++) {
            if (list->nTasks[j] < list->nTasks[min])
                min = j;
        }
        swapSliceListApps(list, i, min);
    }
}

void calcSlices(sliceListT list)
{
    unsigned long total, perApp, ideal, perCPU;
    int app;

    sortSliceListByTasks(list);
    total = (list->period * list->limitPerCPU * list->nCPUs) / 100.0;
    perCPU = list->period * list->limitPerCPU / 100.0;
    for (app = 0; app < list->nApps; app++) {
        perApp = total / (list->nApps - app);
        ideal = perCPU * MIN(list->nTasks[app], list->nCPUs);
        list->slice[app] = MIN(ideal, perApp);
        total -= list->slice[app];
    }
}

void freeSliceList(sliceListT list)
{
    free(list);
}
