#include "fifo.h"
#define _XOPEN_SOURCE 500

int createFIFO(char *filename)
{
    int ret = -1;

    ret = mkfifo(filename, 0600);

    if (ret < 0)
        fprintf(stderr, "Erro ao criar named pipe em %s.\n", filename);

    return ret;
}

void getFIFOName(char *dest, pid_t pid, char *type)
{
    char suffix[MAXFIFONAME];

    strcpy(dest, FIFOPREFIX);
    sprintf(suffix, "%d.%s", pid, type);
    strcat(dest, suffix);
}

void setupClientFIFOs()
{
    char *filename;
    pid_t pid;
    int fd;

    pid = getpid();
    filename = malloc(MAXFIFONAME * sizeof(*filename));

    getFIFOName(filename, pid, "out");
    if (createFIFO(filename) == 0) {
        fd = open(filename, O_WRONLY);
        if (fd < 0)
            fprintf(stderr, "Erro ao abrir %s para escrita.\n", filename);
        else {
            close(1);
            fcntl(fd, F_DUPFD, 1);
            close(fd);
        }
    }

    getFIFOName(filename, pid, "err");
    if (createFIFO(filename) == 0) {
        fd = open(filename, O_WRONLY);
        if (fd < 0)
            fprintf(stderr, "Erro ao abrir %s para escrita.\n", filename);
        else {
            close(2);
            fcntl(fd, F_DUPFD, 2);
            close(fd);
        }
    }

    free(filename);
}

void printOpenErr(char *filename)
{
    fprintf(stderr, "Erro ao abrir %s para leitura: ", filename);
    switch (errno) {
    case ENOENT:
        fprintf(stderr, "arquivo não existe");
        break;
    default:
        fprintf(stderr, "código %d", errno);
    }
    fprintf(stderr, ".\n");
}

/* Tenta abrir o FIFO até por 1 segundo, esperando ser criado pelo cliente */
int openFIFO(const char *path)
{
    int i, fd;

    fd = -1;
    i = 0;
    do {
        fd = open(path, O_RDONLY | O_NONBLOCK);
        i++;
        usleep(1000);
    } while (fd < 0 && errno == ENOENT && i < 1000);

    return fd;
}

void getFIFOs(int *fdOut, int *fdErr, pid_t pid)
{
    char *filename;

    filename = malloc(MAXFIFONAME * sizeof(char));

    getFIFOName(filename, pid, "out");
    *fdOut = getFIFO(filename);
    getFIFOName(filename, pid, "err");
    *fdErr = getFIFO(filename);

    free(filename);
}

int getFIFO(char *filename)
{
    int fd;

    fd = openFIFO(filename);
    if (fd < 0)
        printOpenErr(filename);

    return fd;
}

void readFIFOs(int fdOut, int fdErr)
{
    char ch;
    ssize_t retOut = -1;
    ssize_t retErr = -1;

    do {
        errno = 0;
        if (fdOut >= 0) {
            retOut = read(fdOut, &ch, 1);
            if (retOut == 1) {
                printf("%c", ch);
                fflush(stdout);
            }
        }
        if (fdErr >= 0) {
            retErr = read(fdErr, &ch, 1);
            if (retErr == 1) {
                fprintf(stderr, "%c", ch);
                fflush(stderr);
            }
        }
        usleep(1000);
    } while (retOut > 0 || retErr > 0 || errno == EAGAIN);
}

void removeFIFOs(pid_t pid)
{
    char *filename;

    sleep(1);
    filename = malloc(MAXFIFONAME * sizeof(char));

    getFIFOName(filename, pid, "out");
    removeFIFO(filename);
    getFIFOName(filename, pid, "err");
    removeFIFO(filename);

    free(filename);
}

void removeFIFO(char *filename)
{
    int fd;

    fd = openFIFO(filename);
    if (fd < 0)
        printOpenErr(filename);
    shutdown(fd, SHUT_WR);
    close(fd);
    unlink(filename);
}
