/*******************************************************************************
 * 
 * Arquivo com método que monitora o tempo de execução dos processos.
 *
 ******************************************************************************/

#include"monitor.h"

extern struct processDataStruct *processList;
extern sliceListT sliceList;

/** Retorna o tempo de execução de um determinado processo */
long int getElapsedTime(struct processDataStruct *process)
{
    FILE *file;
    char line[255];
    char fileName[255];
    int intTemp;
    long unsigned int luintTemp;
    char charArrayTemp[255];
    char charTemp;
    long unsigned int systemTime = 0;
    long unsigned int userTime = 0;
    long unsigned int systemTimeTmp;
    long unsigned int userTimeTmp;
    struct pidDataStruct *childrenList;

    sem_wait(&process->childListSem);

    childrenList = process->childrenList;
    /* Avoiding the error below 
     * TODO can we remove the next 'if'?
     */
    while (childrenList == NULL)
        usleep(10000);
    if (childrenList == NULL) {
#ifdef DEBUG
        printf("\nErro ao abrir arquivo de estatística do processo %d...", process->pid);
#endif
        sem_post(&process->childListSem);
        return -1;
    }
    while (childrenList != NULL) {
        sprintf(fileName, "/proc/%d/stat", childrenList->pid);  /*campos 14 e 15 - user e systemtime */
        if ((file = fopen(fileName, "r")) == NULL) {
            if (process->pid == childrenList->pid) {
#ifdef DEBUG
                printf
                    ("\nErro ao abrir arquivo de estatística do processo %d...",
                     childrenList->pid);
#endif
                sem_post(&process->childListSem);
                return -1;
            } else {
                childrenList = childrenList->next;
            }
        } else {
            fgets(line, 255, file);
            fclose(file);
            sscanf(line,
                   "%d %s %c %d %d %d %d %d %lu %lu %lu %lu %lu %lu %lu ",
                   &intTemp, charArrayTemp, &charTemp, (int *) &intTemp,
                   (int *) &intTemp, (int *) &intTemp, (int *) &intTemp,
                   (int *) &intTemp, &luintTemp, &luintTemp, &luintTemp,
                   &luintTemp, &luintTemp, &userTimeTmp, &systemTimeTmp);

            userTime += userTimeTmp;
            systemTime += systemTimeTmp;

            childrenList = childrenList->next;
        }
    }

    sem_post(&process->childListSem);
    return (systemTime + userTime) * JIFFY;     /* Retorna resultado em milisegundos */
}

void *monitorProcessTree()
{
    struct processDataStruct *proc;
    struct processDataStruct *curFather;
    char search[36];
    struct processStruct *procList;
    struct processStruct *node;
    FILE *f;
    struct pidDataStruct *child;
    struct pidDataStruct *child_prev;
    int checked;
    char needToUpdateSlices;

    /* Como evitar que o pstree apareça na lista de processos? */
    sprintf(search, "pstree -l -n -p -A %d", getpid());

    while (1) {
        f = popen(search, "r");
        procList = getPids(f);
        pclose(f);
        node = procList;
        curFather = NULL;
        needToUpdateSlices = 0;

        while (node != NULL) {
            if (node->father == 1) {
                proc = getProcessFromPid(node->pid);
                if (proc != NULL)
                    curFather = proc;
                else
                    curFather = NULL;
            } else {
                if (curFather != NULL) {
                    sem_wait(&curFather->childListSem);
                    child_prev = curFather->childrenList;
                    child = curFather->childrenList->next;
                    checked = 0;
                    while (child != NULL) {
                        if (child->pid == node->pid) {
                            checked = 1;
                            break;
                        }
                        child_prev = child;
                        child = child->next;
                    }
                    if (checked == 0) {
                        struct pidDataStruct *newChild;

                        newChild = (struct pidDataStruct *)
                            malloc(sizeof(struct pidDataStruct));
                        newChild->pid = node->pid;
                        newChild->next = NULL;
                        child_prev->next = newChild;
                        curFather->num_child++;
                        setSliceListNTasks(sliceList, curFather->pid, curFather->num_child);
                        if (!needToUpdateSlices)
                            needToUpdateSlices = 1;
                    }
                    sem_post(&curFather->childListSem);
                }
            }

            node = node->next;
            free(procList);
            procList = node;
        }
        if (needToUpdateSlices) {
            needToUpdateSlices = 0;
            updateSlices(processList, sliceList);
        }
        sleep(1);
    }

    return NULL;
}

struct processStruct *getPids(FILE * f)
{
    char c;
    char pid[6];
    struct processStruct *procList = NULL;
    struct processStruct *procListTail = NULL;
    struct processStruct *node = NULL;
    int i;
    int fatherPos = -1;
    int curPos = 1;
    int res;
    int seek = 0;
    int isFather = 0;

    res = fread(&c, sizeof(char), 1, f);
    while (res != 0) {
        /* The point where child is created was found. */
        if ((fatherPos == -1) && (c == '+')) {
            fatherPos = curPos;
            seek = 1;
            res = fread(&c, sizeof(char), 1, f);
        }
        /* Verifying if the father was found */
        if (seek == 1) {
            if (c != ' ')
                isFather = 1;
            else
                isFather = 0;
            seek = 0;
        }
        /* The pid belongs to a thread, not to a process. */
        if (c == '{') {
            while (c != '(') {
                fread(&c, sizeof(char), 1, f);
                curPos++;
            }
        } else {
            /* The beginning of a process' pid. */
            if (c == '(') {
                i = 0;
                while (c != ')') {
                    fread(&c, sizeof(char), 1, f);
                    pid[i++] = c;
                    curPos++;
                }
                pid[i] = '\0';
                node = (struct processStruct *) malloc(sizeof(struct processStruct));
                node->pid = atoi(pid);
                node->father = isFather;
                isFather = 0;
                node->next = NULL;
                if (procList == NULL) {
                    procList = node;
                } else {
                    procListTail->next = node;
                }
                procListTail = node;
            } else {
                if (c == '\n') {
                    int i;
                    for (i = 0; i < fatherPos; i++)
                        fread(&c, sizeof(char), 1, f);
                    seek = 1;
                }
            }
        }
        res = fread(&c, sizeof(char), 1, f);
        curPos++;
    }

    return procList;
}
