#include "../calcSlices.h"
#include "test.h"
#include <stdio.h>

void testListCreation()
{
    sliceListT list;

    list = NULL;

    list = createSliceList(1, 2, 3);
    ASSERT(list != NULL, "list não pode ser NULL");
    ASSERT(list->nCPUs == 1, "nCPUs deve ser 1");
    ASSERT(list->limitPerCPU == 2, "limitPerCPU deve ser 2");
    ASSERT(list->period == 3, "period deve ser 3");
    ASSERT(list->nApps == 0, "número de aplicações é 0");

    freeSliceList(list);
}

void testAddAnApp()
{
    sliceListT list;
    unsigned long slice;

    list = createSliceList(100, 1, 100);
    addAppToSliceList(list, 123, 1);
    ASSERT(list->nApps == 1, "Após inserção, número de aplicações deve ser 1");
    calcSlices(list);
    slice = getSliceFromSliceList(list, 123);
    ASSERT_EQUALS(1, slice, "Aplicação adicionada contém slice correto");

    freeSliceList(list);
}

void testAddThreeApps()
{
    sliceListT list;
    unsigned long slice;

    list = createSliceList(100, 1, 100);
    addAppToSliceList(list, 4, 1);
    ASSERT(list->nApps == 1, "Número de aplicações deve ser 1");
    addAppToSliceList(list, 5, 2);
    ASSERT(list->nApps == 2, "Número de aplicações deve ser 2");
    addAppToSliceList(list, 6, 3);
    ASSERT(list->nApps == 3, "Número de aplicações deve ser 3");

    calcSlices(list);
    slice = getSliceFromSliceList(list, 4);
    ASSERT(slice == 1, "Primeira app com slice correto");
    slice = getSliceFromSliceList(list, 5);
    ASSERT(slice == 2, "Segunda app com slice correto");
    slice = getSliceFromSliceList(list, 6);
    ASSERT(slice == 3, "Terceira app com slice correto");

    freeSliceList(list);
}

void removeFirstApp()
{
    sliceListT list;
    unsigned long slice;

    list = createSliceList(100, 100, 100);
    ASSERT(list->nApps == 0, "Nenhum aplicativo");
    addAppToSliceList(list, 1, 1);
    ASSERT(list->nApps == 1, "Pid 1 adicionado");
    addAppToSliceList(list, 2, 2);
    ASSERT(list->nApps == 2, "Pid 2 adicionado");
    addAppToSliceList(list, 3, 3);
    ASSERT(list->nApps == 3, "Pid 3 adicionado");
    calcSlices(list);

    removeAppFromSliceList(list, 1);
    ASSERT(list->nApps == 2, "2 aplicativos após remoção de pid 1");
    slice = getSliceFromSliceList(list, 2);
    ASSERT(slice == 200, "Pid 2 encontrado");
    slice = getSliceFromSliceList(list, 3);
    ASSERT(slice == 300, "Pid 3 encontrado");

    freeSliceList(list);
}

void removeLastApp()
{
    sliceListT list;
    unsigned long slice;

    list = createSliceList(100, 100, 100);
    ASSERT(list->nApps == 0, "Nenhum aplicativo");
    addAppToSliceList(list, 1, 1);
    ASSERT(list->nApps == 1, "Pid 1 adicionado");
    addAppToSliceList(list, 2, 2);
    ASSERT(list->nApps == 2, "Pid 2 adicionado");
    addAppToSliceList(list, 3, 3);
    ASSERT(list->nApps == 3, "Pid 3 adicionado");
    calcSlices(list);

    removeAppFromSliceList(list, 3);
    ASSERT(list->nApps == 2, "2 aplicativos após remoção de pid 3");
    slice = getSliceFromSliceList(list, 1);
    ASSERT(slice == 100, "Pid 1 encontrado");
    slice = getSliceFromSliceList(list, 2);
    ASSERT(slice == 200, "Pid 2 encontrado");

    freeSliceList(list);
}

void oneTaskShouldUseAllTheSlicePerCPU()
{
    sliceListT list;
    short int nCPUs = 8;
    char limitPerCPU = 50;
    unsigned long period = 1000;
    unsigned long slice;

    list = createSliceList(nCPUs, limitPerCPU, period);
    addAppToSliceList(list, 100, 1);
    calcSlices(list);
    slice = getSliceFromSliceList(list, 100);
    ASSERT_EQUALS(500, (int) slice, "Slice dever ser metade do período");

    freeSliceList(list);
}

void shouldPassKnownCase1()
{
    sliceListT list;
    unsigned long slice;

    list = createSliceList(16, 25, 1000);
    addAppToSliceList(list, 1, 1);
    addAppToSliceList(list, 2, 10);
    addAppToSliceList(list, 3, 2);
    addAppToSliceList(list, 4, 1);
    addAppToSliceList(list, 5, 1);
    addAppToSliceList(list, 6, 20);
    addAppToSliceList(list, 7, 1);
    addAppToSliceList(list, 8, 3);
    calcSlices(list);
    slice = getSliceFromSliceList(list, 1);
    ASSERT_EQUALS(250, slice, "slice de pid 1");
    slice = getSliceFromSliceList(list, 2);
    ASSERT_EQUALS(875, slice, "slice de pid 2");
    slice = getSliceFromSliceList(list, 3);
    ASSERT_EQUALS(500, slice, "slice de pid 3");
    slice = getSliceFromSliceList(list, 4);
    ASSERT_EQUALS(250, slice, "slice de pid 4");
    slice = getSliceFromSliceList(list, 5);
    ASSERT_EQUALS(250, slice, "slice de pid 5");
    slice = getSliceFromSliceList(list, 6);
    ASSERT_EQUALS(875, slice, "slice de pid 6");
    slice = getSliceFromSliceList(list, 7);
    ASSERT_EQUALS(250, slice, "slice de pid 7");
    slice = getSliceFromSliceList(list, 8);
    ASSERT_EQUALS(750, slice, "slice de pid 8");

    freeSliceList(list);
}

void shouldPassKnownCase2()
{
    sliceListT list;
    unsigned long slice;

    list = createSliceList(2, 50, 1000);
    addAppToSliceList(list, 1, 1);
    addAppToSliceList(list, 2, 1);
    addAppToSliceList(list, 3, 1);
    addAppToSliceList(list, 4, 1);
    calcSlices(list);
    slice = getSliceFromSliceList(list, 1);
    ASSERT_EQUALS(250, slice, "Slice de pid 1");
    slice = getSliceFromSliceList(list, 2);
    ASSERT_EQUALS(250, slice, "Slice de pid 2");
    slice = getSliceFromSliceList(list, 3);
    ASSERT_EQUALS(250, slice, "Slice de pid 4");
    slice = getSliceFromSliceList(list, 4);
    ASSERT_EQUALS(250, slice, "Slice de pid 3");

    freeSliceList(list);
}

/* Testes internos, não de uso real */
void shouldSortByTasks()
{
    sliceListT list;

    list = createSliceList(1, 2, 3);
    addAppToSliceList(list, 0, 6);
    addAppToSliceList(list, 1, 8);
    addAppToSliceList(list, 2, 5);
    addAppToSliceList(list, 3, 7);
    calcSlices(list);

    /* Parte a ser usada somente para teste */
    ASSERT(list->nTasks[0] == 5, "Primeira app com nTasks correto");
    ASSERT(list->pid[0] == 2, "Primeira app com pid correto");
    ASSERT(list->nTasks[1] == 6, "Segunda app com nTasks correto");
    ASSERT(list->pid[1] == 0, "Segunda app com pid correto");
    ASSERT(list->nTasks[2] == 7, "Terceira app com nTasks correto");
    ASSERT(list->pid[2] == 3, "Terceira app com pid correto");
    ASSERT(list->nTasks[3] == 8, "Quarta app com nTasks correto");
    ASSERT(list->pid[3] == 1, "Quarta app com pid correto");

    freeSliceList(list);
}

int main(int argc, char **argv)
{
    printf("---- Testes de calcSlices.c ----\n");
    printf("Criação da lista de slices...\n");
    testListCreation();
    printf("Adicionando a primeira aplicação...\n");
    testAddAnApp();
    printf("Adicionando três aplicações...\n");
    testAddThreeApps();
    printf("Removendo primeiro aplicativo...\n");
    removeFirstApp();
    printf("Removendo último aplicativo...\n");
    removeLastApp();
    printf("Ordenação por número de tarefas por aplicação...\n");
    shouldSortByTasks();
    printf("Uma tarefa deve consumir o limite de 1 CPU...\n");
    oneTaskShouldUseAllTheSlicePerCPU();
    printf("Calculando slices do caso conhecido 1...\n");
    shouldPassKnownCase1();
    printf("Calculando slices do caso conhecido 2...\n");
    shouldPassKnownCase2();
    printf("---- Resultado ----\n");
    SUMMARY;
    return 0;
}
