#include <stdio.h>
#include <errno.h>
#include <sched.h>
#include <unistd.h>
#include <sys/resource.h>
#include <bits/xopen_lim.h>

void printSchedErr()
{
    switch errno {
    case EPERM:
        printf("Usuario sem permissão.\n");
        break;
    case EINVAL:
        printf("Parâmetros inválidos.\n");
        break;
    case ESRCH:
        printf("Processo nao encontrado.\n");
        break;
    case EACCES:
        printf("No privileges for such a low nice value.\n");
        break;
    default:
        printf("Unknown error code: %d.\n", errno);
    }
}

void printNiceErr()
{
    switch (errno) {
    case ESRCH:
        printf("No process found.\n");
        break;
    case EINVAL:
        printf("Invalid parameter.\n");
        break;
    case EPERM:
        printf("user ID doesn't match.\n");
        break;
    default:
        printf("Unknown error.\n");
    }
}

void printSchedInfo()
{
    int sched, nice;
    struct sched_param param;

    printf("Status of pid %d:\n", getpid());
    sched = sched_getscheduler(0);
    switch (sched) {
    case SCHED_RR:
        printf("\tSCHED_RR\n");
        sched_getparam(0, &param);
        printf("\tsched_priority: %d\n", param.sched_priority);
        break;
    case SCHED_OTHER:
        printf("\tSCHED_OTHER\n");
        errno = 0;
        nice = getpriority(PRIO_PROCESS, 0);
        if (nice == -1 && errno != 0) {
            printf("getpriority error: ");
            printNiceErr();
        } else
            printf("\tnice: %d\n", nice);
        break;
    default:
        printf("\tSCHED Desconhecido\n");
    }
}

int setMaxRT()
{
    int maxPrio, minPrio, i;
    int res = -1;
    struct sched_param params;

    maxPrio = sched_get_priority_max(SCHED_RR);
    minPrio = sched_get_priority_min(SCHED_RR);

    printf("Max SCHED_RR prio: %d\n", maxPrio);
    printf("Min SCHED_RR prio: %d\n", minPrio);

    for (i = maxPrio; i >= minPrio; i--) {
        printf("SCHED_RR, priority = %d: ", i);
        params.sched_priority = i;
        res = sched_setscheduler(getpid(), SCHED_RR, &params);
        if (res != -1) {
            printf("success!\n");
            return 1;
        } else
            printSchedErr();
    }
    return 0;
}

void setMaxOther()
{
    int minNice, maxNice, i;
    int res = -1;
    struct sched_param params;

    minNice = -NZERO;
    maxNice = getpriority(PRIO_PROCESS, 0);

    printf("Min SCHED_OTHER nice: %d\n", minNice);
    printf("Max SCHED_OTHER nice: %d (current)\n", maxNice);

    for (i = minNice; i < maxNice; i++) {
        printf("SCHED_OTHER, nice = %d: ", i);
        params.sched_priority = i;
        res = setpriority(PRIO_PROCESS, 0, i);
        if (res == -1)
            printSchedErr();
        else {
            printf("success!\n");
            return;
        }
    }
    printf("Initial priority (nice) is the maximium allowed.\n");
}

int main(int argc, char **argv)
{
    if (!setMaxRT())
        setMaxOther();

    printSchedInfo();
    printf("Check top while I sleep a minute...\n");
    sleep(60);

    return 0;
}
