/*
 * Limiting CPU using CPUReserve
 * Cadu 2009 (TCC)
 */

#include "CPU.h"
#include "CPUReserve.h"

void CPU::generateVars()
{
	stringstream aux;
	CPUReserve cpur(PORT);

	if (queryString.find("OK") != queryString.end())
		applyChanges();

	if (!cpur.updateStatus())
		exit(1);

	aux << getCheckboxes(cpur.getNumberOfCPUs(), cpur.getMask());
	setVar("CPU_CORES", aux.str());

	aux.str("");
	aux << cpur.getCPULimit();
	setVar("CPU_LIMIT", aux.str());
}


void CPU::applyChanges()
{
	int newCPULimit;
	int newBitmask;
	bool *newActiveCores;
	CPUReserve cpur(PORT);

	cpur.updateStatus();

	sscanf(queryString["cpuLimit"].c_str(), "%d", &newCPULimit);
	newActiveCores = getActiveCoresFromHtml(cpur.getNumberOfCPUs());
	newBitmask = toBitmask(newActiveCores, cpur.getNumberOfCPUs());

	delete [] newActiveCores;
	cpur.adapt(newCPULimit, newBitmask);
}


bool* CPU::getActiveCoresFromHtml(int numberOfCPUs)
{
	stringstream htmlName;
	bool *myActiveCores = new bool[numberOfCPUs];

	for (int i = 0; i < numberOfCPUs; i++) {
		htmlName.str("");
		htmlName << "cpu" << i;
		if (queryString.find(htmlName.str().c_str()) != queryString.end()
		&& queryString[htmlName.str()] == "1") {
			myActiveCores[i] = true;
		}
		else {
			myActiveCores[i] = false;
		}
	}
	return myActiveCores;
}


bool* CPU::fromBitmask(int bitmask, int numberOfCPUs)
{
	bool *active;
	int i;

	active = new bool[numberOfCPUs];

	for (i = 0; i < numberOfCPUs; i++) {
		if ( ( bitmask & (1 << i)) != 0 ) {
			active[i] = true;
		}
		else {
			active[i] = false;
		}
	}
	return active;
}


int CPU::toBitmask(bool* active, int numberOfCPUs)
{
	int i;
	int bitmask = 0;

	for (i = 0; i < numberOfCPUs; i++)
		if (active[i])
			bitmask += pow(2, i);
	return bitmask;
}


string CPU::getCheckboxes(int numberOfCPUs, int mask)
{
	stringstream checkboxes;
	bool *activeCores;

	activeCores = fromBitmask(mask, numberOfCPUs);

	for (int i = 0; i < numberOfCPUs; i++) {
		checkboxes << "<input type=\"checkbox\" name=\"cpu" << i << "\" ";
		checkboxes << "value=\"1\" ";
		if (activeCores[i]) {
			checkboxes << "checked=\"true\" ";
		}
		checkboxes << "/>" << i << endl;
	}

	delete [] activeCores;
	return checkboxes.str();
}
