#include "CPUReserve.h"

CPUReserve::~CPUReserve()
{
	delete processList;
}


void CPUReserve::adapt(int newCPULimit, int newMask)
{
	stringstream cmd;
	int status;

	updateStatus();

	if (mask != newMask || cpuLimit != newCPULimit) {
		cmd << "./adapt 127.0.0.1:" << port << " " << newMask;
		cmd << " " << newCPULimit << " 2>>" << LOG_FILE;
		status = system(cmd.str().c_str());
	}
}


bool CPUReserve::updateStatus()
{
	ostringstream cmd;
	FILE *pstatus;
	char buf[32];
	string status;
	int exitStatus;

	cmd << "./adapt 127.0.0.1:" << port << " status";
	exitStatus = system(cmd.str().c_str());
	pstatus = popen(cmd.str().c_str(), "r");

	if (exitStatus != 0 || pstatus == NULL) {
		string msg = "Error connecting to CPUReserve server while executing: " + cmd.str();
		log.write(msg);
		return false;
	}

	while (fgets(buf, 32, pstatus) != NULL) {
		status.append(buf);
	}
	pclose(pstatus);
	parseStatus(status);

	return true;
}


/*
	Format:
	CPU: [mask] [total number of CPUs] [limit percentage]
	Processes:
	-[parent_1 pid]
	--[child_1 pid]
	--[child 2 pid]
	-[parent_2 pid]
	...
*/
void CPUReserve::parseStatus(string status)
{
	if (sscanf(status.c_str(), "CPU: %d %d %d",
	&mask, &numberOfCPUs, &cpuLimit) != 3) {
		string msg = "couldn't parse CPU status line '" + status;
		log.write(msg);
	}
	if (processList == NULL)
		processList = new ProcessList();

	string procLine;
	int pos = 0;
	int pid;
	pos = status.find_first_of('-', pos);
	while (pos != (int) string::npos) {
		procLine = getLine(status, pos);
		/* Child */
		if(sscanf(procLine.c_str(), "--%d", &pid) == 1)
			processList->addLastParentChild(pid);
		/* Parent */
		else if(sscanf(procLine.c_str(), "-%d", &pid) == 1)
			processList->addNewParent(pid);
		pos = status.find_first_of('-', pos+2);
	}
}


/* Private */
string CPUReserve::getLine(string status, int start)
{
	int end;
	int length;

	end = status.find_first_of('\n', start);
	length = end - start;

	return status.substr(start, length);
}
