#include <stdio.h>
#include <sstream>
#include <iostream>
#include <stdlib.h>
#include "ProcessList.h"

/*
   Process methods
 */
Process::Process() : cpu(-1), father(false), mem("-1"), name(""), pid(-1)
{
	children = new vector<Process*>();
}


Process::~Process()
{
	vector<Process*>::iterator child = children->begin();
	for (; child < children->end(); child++)
		delete &(**child);
	delete children;
}


/*
   ProcessList Methods
 */

Process* Process::getNewProcess()
{
	Process *p = new Process();
	return p;
}


/*
   ProcessList methods
 */

ProcessList::ProcessList()
{
	processes = new vector<Process*>();
}


ProcessList::~ProcessList()
{
	vector<Process*>::iterator parent = processes->begin();
	for (; parent < processes->end(); parent++)
		delete &(**parent);
	delete processes;
}


void ProcessList::addNewParent(int pid)
{
	Process *parent = Process::getNewProcess();
	parent->setPid(pid);
	parent->setFather();
	processes->insert(processes->begin(), parent);
}


void ProcessList::addLastParentChild(int pid)
{
	Process *child = Process::getNewProcess();
	child->setPid(pid);
	processes->front()->addChild(child);
}


void ProcessList::parseTopOutput(int numberOfCPUs)
{
	if (processes->size() == 0)
		return;

	int pid, cpu;
	char mem[8], aux[16], name[16];
	string cmd = getTopCmd();
	cout << cmd << endl;
	FILE *top = popen(cmd.c_str(), "r");
	Process *process = NULL;

	while (fscanf(top, "%d %s %s %s %s %s %s %s %d %s %s %s\n",
	&pid, aux, aux, aux, aux, mem, aux, aux, &cpu, aux, aux, name) == 12) {
		process = getProcessByPid(pid);
		if (process != NULL) {
			process->setCPU(cpu/numberOfCPUs);
			process->setMem(mem);
			process->setName(name);
		}
	}

	pclose(top);
}


string ProcessList::getTopCmd()
{
	vector<Process*>::iterator parentIt = processes->begin();
	Process *parent = NULL;
	vector<Process*> *children;
	vector<Process*>::iterator childIt;
	Process *child = NULL;
	stringstream pids;
	int nPids = 0;

	while (parentIt < processes->end()) {
		parent = *parentIt;
		pids << parent->getPid();
		nPids++;
		children = parent->getChildren();
		for (childIt = children->begin(); childIt < children->end(); childIt++) {
			child = *childIt;
			pids << "," << child->getPid();
			nPids++;
		}
		if ( *parent != *(processes->back()) )
			pids << ",";
		parentIt++;
	}

	ostringstream cmd;
	cmd << "top -b -n 1 -p "<< pids.str() << " | " <<
		"tail -n " << (nPids + 1);

	return cmd.str();
}


Process* ProcessList::getProcessByPid(int pid)
{
	vector<Process*>::iterator parentIt = processes->begin();
	Process *parent;
	vector<Process*> *children;
	vector<Process*>::iterator childIt;
	Process *child;

	while (parentIt < processes->end()) {
		parent = *parentIt;
		if (parent->getPid() == pid)
			return &(*parent);
		children = parent->getChildren();
		childIt = children->begin();
		while (childIt < children->end()) {
			child = *childIt;
			if (child->getPid() == pid)
				return &(*child);
			childIt++;
		}
		parentIt++;
	}
	return NULL;
}


void ProcessList::killApp(int nApp)
{
	vector<Process*>::iterator parent = processes->begin();
	vector<Process*>::iterator child;
	vector<Process*> *children;
	ostringstream cmd;

	cmd << "kill ";

	parent = processes->begin() + nApp - 1;
	if (parent > processes->end() || parent < processes->begin()) {
		return;
	}

	cmd << (*parent)->getPid() << " ";
	children = (*parent)->getChildren();
	for (child = children->begin(); child != children->end(); child++)
		cmd << (*child)->getPid() << " ";

	system(cmd.str().c_str());
	processes->erase(parent);
}
