////////////////////////////////////////////////////////////////////////////////////////////
// Project: NCC InteGrade Interface                           //
// Developed by: Giulian Dalton Luz & Rogerio Guaraci dos Santos              //
// Date: 8/12/2003                                    //
// Version: Beta 1                                    //
// Module Description: ServerSocket.cpp. Implements the socket server.                //
////////////////////////////////////////////////////////////////////////////////////////////

#include "ServerSocket.h"
#include "SocketException.h"

ServerSocket::ServerSocket ( int port )
{
	if ( ! socket.create() ) {
		throw SocketException ( "Could not create server socket." );
	}

	if ( ! socket.bind ( port ) ) {
		throw SocketException ( "Could not bind to port." );
	}

	if ( ! socket.listen() ) {
		throw SocketException ( "Could not listen to socket." );
	}
}


const ServerSocket& ServerSocket::operator << ( const std::string& s ) const
{
	if ( ! socket.send ( s ) ) {
		throw SocketException ( "Could not write to socket." );
	}

	return *this;
}


const ServerSocket& ServerSocket::operator >> ( std::string& s ) const
{
	if ( ! socket.recv ( s ) ) {
		throw SocketException ( "Could not read from socket." );
	}

	return *this;
}


void ServerSocket::accept()
{
	if ( ! socket.accept() ) {
		throw SocketException ( "Could not accept socket." );
	}
}


void ServerSocket::close()
{
	socket.close();
}


void ServerSocket::closeSocket()
{
	socket.closeConnection();
}
